"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

var _overflow = require("./overflow");

var _overflow2 = _interopRequireDefault(_overflow);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var json_parse = function () {

    var state;
    var stack;
    var container;
    var key;
    var value;
    var escapes = {
        "\\": "\\",
        "\"": "\"",
        "/": "/",
        "t": "\t",
        "n": "\n",
        "r": "\r",
        "f": "\f",
        "b": "\b"
    };
    var string = {
        go: function go() {
            state = "ok";
        },
        firstokey: function firstokey() {
            key = value;
            state = "colon";
        },
        okey: function okey() {
            key = value;
            state = "colon";
        },
        ovalue: function ovalue() {
            state = "ocomma";
        },
        firstavalue: function firstavalue() {
            state = "acomma";
        },
        avalue: function avalue() {
            state = "acomma";
        }
    };
    var number = {
        go: function go() {
            state = "ok";
        },
        ovalue: function ovalue() {
            state = "ocomma";
        },
        firstavalue: function firstavalue() {
            state = "acomma";
        },
        avalue: function avalue() {
            state = "acomma";
        }
    };
    var action = {

        "{": {
            go: function go() {
                stack.push({ state: "ok" });
                container = {};
                state = "firstokey";
            },
            ovalue: function ovalue() {
                stack.push({ container: container, state: "ocomma", key: key });
                container = {};
                state = "firstokey";
            },
            firstavalue: function firstavalue() {
                stack.push({ container: container, state: "acomma" });
                container = {};
                state = "firstokey";
            },
            avalue: function avalue() {
                stack.push({ container: container, state: "acomma" });
                container = {};
                state = "firstokey";
            }
        },
        "}": {
            firstokey: function firstokey() {
                var pop = stack.pop();
                value = container;
                container = pop.container;
                key = pop.key;
                state = pop.state;
            },
            ocomma: function ocomma() {
                var pop = stack.pop();
                container[key] = value;
                value = container;
                container = pop.container;
                key = pop.key;
                state = pop.state;
            }
        },
        "[": {
            go: function go() {
                stack.push({ state: "ok" });
                container = [];
                state = "firstavalue";
            },
            ovalue: function ovalue() {
                stack.push({ container: container, state: "ocomma", key: key });
                container = [];
                state = "firstavalue";
            },
            firstavalue: function firstavalue() {
                stack.push({ container: container, state: "acomma" });
                container = [];
                state = "firstavalue";
            },
            avalue: function avalue() {
                stack.push({ container: container, state: "acomma" });
                container = [];
                state = "firstavalue";
            }
        },
        "]": {
            firstavalue: function firstavalue() {
                var pop = stack.pop();
                value = container;
                container = pop.container;
                key = pop.key;
                state = pop.state;
            },
            acomma: function acomma() {
                var pop = stack.pop();
                container.push(value);
                value = container;
                container = pop.container;
                key = pop.key;
                state = pop.state;
            }
        },
        ":": {
            colon: function colon() {
                if (Object.hasOwnProperty.call(container, key)) {
                    throw new SyntaxError("Duplicate key '" + key + "\"");
                }
                state = "ovalue";
            }
        },
        ",": {
            ocomma: function ocomma() {
                container[key] = value;
                state = "okey";
            },
            acomma: function acomma() {
                container.push(value);
                state = "avalue";
            }
        },
        "true": {
            go: function go() {
                value = true;
                state = "ok";
            },
            ovalue: function ovalue() {
                value = true;
                state = "ocomma";
            },
            firstavalue: function firstavalue() {
                value = true;
                state = "acomma";
            },
            avalue: function avalue() {
                value = true;
                state = "acomma";
            }
        },
        "false": {
            go: function go() {
                value = false;
                state = "ok";
            },
            ovalue: function ovalue() {
                value = false;
                state = "ocomma";
            },
            firstavalue: function firstavalue() {
                value = false;
                state = "acomma";
            },
            avalue: function avalue() {
                value = false;
                state = "acomma";
            }
        },
        "null": {
            go: function go() {
                value = null;
                state = "ok";
            },
            ovalue: function ovalue() {
                value = null;
                state = "ocomma";
            },
            firstavalue: function firstavalue() {
                value = null;
                state = "acomma";
            },
            avalue: function avalue() {
                value = null;
                state = "acomma";
            }
        }
    };

    function debackslashify(text) {

        return text.replace(/\\(?:u(.{4})|([^u]))/g, function (ignore, b, c) {
            return b ? String.fromCharCode(parseInt(b, 16)) : escapes[c];
        });
    }

    return function (source, reviver) {

        var result;
        var tx = /^[\u0020\t\n\r]*(?:([,:\[\]{}]|true|false|null)|(-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?)|"((?:[^\r\n\t\\\"]|\\(?:["\\\/trnfb]|u[0-9a-fA-F]{4}))*)")/;

        state = "go";

        stack = [];

        try {

            while (true) {
                result = tx.exec(source);
                if (!result) {
                    break;
                }

                if (result[1]) {

                    action[result[1]][state]();
                } else if (result[2]) {
                    value = (0, _overflow2.default)(result[2]) ? result[2] : +result[2];
                    number[state]();
                } else {

                    value = debackslashify(result[3]);
                    string[state]();
                }

                source = source.slice(result[0].length);
            }
        } catch (e) {
            state = e;
        }

        if (state !== "ok" || /[^\u0020\t\n\r]/.test(source)) {
            throw state instanceof SyntaxError ? state : new SyntaxError("JSON");
        }

        return typeof reviver === "function" ? function walk(holder, key) {
            var k;
            var v;
            var val = holder[key];
            if (val && (typeof val === "undefined" ? "undefined" : _typeof(val)) === "object") {
                for (k in value) {
                    if (Object.prototype.hasOwnProperty.call(val, k)) {
                        v = walk(val, k);
                        if (v !== undefined) {
                            val[k] = v;
                        } else {
                            delete val[k];
                        }
                    }
                }
            }
            return reviver.call(holder, key, val);
        }({ "": value }, "") : value;
    };
}();

exports.default = json_parse;